@@ -38,7 +38,7 @@ s27:  ia[7] = 3, ja[7] = 2, ar[7] =  2.0; /* a[3,2] =  2 */
 s28:  ia[8] = 2, ja[8] = 3, ar[8] =  5.0; /* a[2,3] =  5 */
 s29:  ia[9] = 3, ja[9] = 3, ar[9] =  6.0; /* a[3,3] =  6 */
 s30:  glp_load_matrix(lp, 9, ia, ja, ar);
-s31:  lpx_simplex(lp);
+s31:  glp_simplex(lp, NULL);
 s32:  z = glp_get_obj_val(lp);
 s33:  x1 = glp_get_col_prim(lp, 1);
 s34:  x2 = glp_get_col_prim(lp, 2);
