@@ -3,7 +3,7 @@
 /***********************************************************************
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
-*  Copyright (C) 2007-2016 Andrew Makhorin, Department for Applied
+*  Copyright (C) 2007-2017 Andrew Makhorin, Department for Applied
 *  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
 *  reserved. E-mail: <mao@gnu.org>.
 *
@@ -21,6 +21,10 @@
 *  along with GLPK. If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #if 1 /* 29/II-2016 by Chris */
 /*----------------------------------------------------------------------
 Subject: Mir cut generation performance improvement
@@ -798,7 +802,11 @@ static int cmir_ineq(const int n, const double a[], const double b,
 
 struct vset { int j; double v; };
 
+#ifndef __WOE__
 static int cmir_cmp(const void *p1, const void *p2)
+#else
+static int __cdecl cmir_cmp(const void *p1, const void *p2)
+#endif
 {     const struct vset *v1 = p1, *v2 = p2;
       if (v1->v < v2->v) return -1;
       if (v1->v > v2->v) return +1;
